@@ -34,7 +34,7 @@ from markdown.blockprocessors import BlockProcessor
 from markdown.extensions import Extension
 from markdown.treeprocessors import Treeprocessor
 from mkdocs.exceptions import PluginError
-from mkdocs_autorefs import AutorefsConfig, AutorefsPlugin
+from mkdocs_autorefs import AutorefsConfig, AutorefsExtension, AutorefsPlugin
 
 from mkdocstrings._internal.handlers.base import BaseHandler, CollectionError, CollectorItem, Handlers
 from mkdocstrings._internal.loggers import get_logger
@@ -316,17 +316,26 @@ class MkdocstringsExtension(Extension):
     It cannot work outside of `mkdocstrings`.
     """
 
-    def __init__(self, handlers: Handlers, autorefs: AutorefsPlugin, **kwargs: Any) -> None:
+    def __init__(
+        self,
+        handlers: Handlers,
+        autorefs: AutorefsPlugin,
+        *,
+        autorefs_extension: bool = False,
+        **kwargs: Any,
+    ) -> None:
         """Initialize the object.
 
         Arguments:
             handlers: The handlers container.
             autorefs: The autorefs plugin instance.
+            autorefs_extension: Whether the autorefs extension must be registered.
             **kwargs: Keyword arguments used by `markdown.extensions.Extension`.
         """
         super().__init__(**kwargs)
         self._handlers = handlers
         self._autorefs = autorefs
+        self._autorefs_extension = autorefs_extension
 
     def extendMarkdown(self, md: Markdown) -> None:  # noqa: N802 (casing: parent method's name)
         """Register the extension.
@@ -336,6 +345,12 @@ class MkdocstringsExtension(Extension):
         Arguments:
             md: A `markdown.Markdown` instance.
         """
+        md.registerExtension(self)
+
+        # Zensical integration: get the current page from the Zensical-specific preprocessor.
+        if "zensical_current_page" in md.preprocessors:
+            self._autorefs.current_page = md.preprocessors["zensical_current_page"]  # type: ignore[assignment]
+
         md.parser.blockprocessors.register(
             AutoDocProcessor(md, handlers=self._handlers, autorefs=self._autorefs),
             "mkdocstrings",
@@ -352,6 +367,9 @@ class MkdocstringsExtension(Extension):
             priority=4,  # Right after 'toc'.
         )
 
+        if self._autorefs_extension:
+            AutorefsExtension(self._autorefs).extendMarkdown(md)
+
 
 # -----------------------------------------------------------------------------
 # The following is only used by Zensical. The goal is to provide temporary
@@ -369,7 +387,7 @@ _default_config: dict[str, Any] = {
 }
 
 
-def _split_configs(markdown_extensions: list[str | dict]) -> tuple[list[str], dict[str, Any]]:
+def _split_configs(markdown_extensions: list[str | dict]) -> tuple[list[str | Extension], dict[str, Any]]:
     # Split markdown extensions and their configs from mkdocs.yml
     mdx: list[str] = []
     mdx_config: dict[str, Any] = {}
@@ -381,7 +399,7 @@ def _split_configs(markdown_extensions: list[str | dict]) -> tuple[list[str], di
                 mdx.append(key)
                 mdx_config[key] = value
                 break  # Only one item per dict
-    return mdx, mdx_config
+    return mdx, mdx_config  # type: ignore[return-value]
 
 
 class _ToolConfig:
@@ -389,6 +407,10 @@ class _ToolConfig:
         self.config_file_path = config_file_path
 
 
+_AUTOREFS = None
+_HANDLERS = None
+
+
 def makeExtension(  # noqa: N802
     *,
     default_handler: str | None = None,
@@ -405,26 +427,65 @@ def makeExtension(  # noqa: N802
     We only support this function being used by Zensical.
     Consider this function private API.
     """
-    mdx, mdx_config = _split_configs(markdown_extensions or [])
-    tool_config = _ToolConfig(config_file_path=config_file_path)
+    global _AUTOREFS  # noqa: PLW0603
+    if _AUTOREFS is None:
+        _AUTOREFS = AutorefsPlugin()
+        _AUTOREFS.config = AutorefsConfig()
+        _AUTOREFS.config.resolve_closest = True
+        _AUTOREFS.config.link_titles = "auto"
+        _AUTOREFS.config.strip_title_tags = "auto"
+        _AUTOREFS.scan_toc = True
+        _AUTOREFS._link_titles = "external"
+        _AUTOREFS._strip_title_tags = False
 
-    handlers_instance = Handlers(
-        theme="material",
-        default=default_handler or _default_config["default_handler"],
-        inventory_project=inventory_project or "Project",
-        inventory_version=inventory_version or "0.0.0",
-        handlers_config=handlers or _default_config["handlers"],
-        custom_templates=custom_templates or _default_config["custom_templates"],
-        mdx=mdx,
-        mdx_config=mdx_config,
-        locale=locale or _default_config["locale"],
-        tool_config=tool_config,
+    global _HANDLERS  # noqa: PLW0603
+    if _HANDLERS is None:
+        mdx, mdx_config = _split_configs(markdown_extensions or [])
+        tool_config = _ToolConfig(config_file_path=config_file_path)
+        mdx.append(AutorefsExtension(_AUTOREFS))
+        _HANDLERS = Handlers(
+            theme="material",
+            default=default_handler or _default_config["default_handler"],
+            inventory_project=inventory_project or "Project",
+            inventory_version=inventory_version or "0.0.0",
+            handlers_config=handlers or _default_config["handlers"],
+            custom_templates=custom_templates or _default_config["custom_templates"],
+            mdx=mdx,
+            mdx_config=mdx_config,
+            locale=locale or _default_config["locale"],
+            tool_config=tool_config,
+        )
+
+        _HANDLERS._download_inventories()
+        register = _AUTOREFS.register_url
+        for identifier, url in _HANDLERS._yield_inventory_items():
+            register(identifier, url)
+
+    return MkdocstringsExtension(
+        handlers=_HANDLERS,
+        autorefs=_AUTOREFS,
+        autorefs_extension=True,
     )
 
-    handlers_instance._download_inventories()
 
-    autorefs = AutorefsPlugin()
-    autorefs.config = AutorefsConfig()
-    autorefs.scan_toc = False
+def _reset() -> None:
+    global _AUTOREFS, _HANDLERS  # noqa: PLW0603
+    _AUTOREFS = None
+    _HANDLERS = None
 
-    return MkdocstringsExtension(handlers=handlers_instance, autorefs=autorefs)
+
+def _get_autorefs() -> dict[str, Any]:
+    if _AUTOREFS:
+        return {
+            "primary": _AUTOREFS._primary_url_map,
+            "secondary": _AUTOREFS._secondary_url_map,
+            "inventory": _AUTOREFS._abs_url_map,
+            "titles": _AUTOREFS._title_map,
+        }
+    return {}
+
+
+def _get_inventory() -> bytes:
+    if _HANDLERS:
+        return _HANDLERS.inventory.format_sphinx()
+    return b""
